// ZoneMinder (A)ctive (R)response (C)amera console view JS file, $Date:
// 2013-08-30 19:52:12 +0000 (Fri, 30 Aug 2013) $, $Revision: 1 $
// Copyright (C) 2013 Anthony Cassandra
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

var ARC_DEBUG_ENABLED = true;

//========================================
// Theory of Operation
//========================================

// The (A)ctive (R)response (C)amera console is a custom view (really set
// of views) for ZoneMInder that fits a the particular usage pattern I
// (Anthony Cassandra) needed.  It gives you basic control of seeing live
// feeds and events, but is tailored toward the most recent events, and
// more importantly, it automatically will play new events and switch live
// feeds to the monitor (a.k.a., camera) where the latest event occurred.
// This is more useful for me for general live monitoring, or when reacting
// to the notification of an event and wanting to explore it.  Becausde it
// automatically updates, when you receive a notifcation of an alarm, going
// to this console should provide a more immediate view of the situation.

//--------------------
// Iframes

// There are 3 iframes that have independent activity and updating.
//
//   o Latest Event Listing Iframe - This is periodically refreshed with
//     the latest events from all monitors.  On a refresh, it may have 
//     no new events since the last time, or we may detect new events.
//     When new events are detected, we will (normally) automatically
//     play the event in the event video iframe and switch the live
//     video iframe feed to the monitor the event was triggered on.
//     The automatic updating of iframes based on events will be
//     be suppressed if we have detected that the user is actively
//     engaged witht the console in a way that auto-updates would
//     be disruptive to them.  On of the ways the user can interact
//     is to click on an event in the event listings, in which case
//     the event video iframe will be changed to display that event.
//
//   o Event Listing Iframe - This is refreshed with the
//     latest events from all monitors each time a new event was found.
//     The user can also use this to browse events and select events.
//     The auto-updating of this iframe with latest events is suspended
//     while the user interacts with this, but resumes when the user
//     interactions are far enough in the past.
//
//   o Event Video Iframe - Used to display the video feed for an event.
//
//   o Live Video Iframe - Used to display a live feed from one of the
//     monitors.  There are also controls that allow the user to manually 
//     select the live video feed from different monitors.  The user can 
//     also choose to view all the monitors in a montage, or can choose to
//     to cycle through all the views.
//
// After a period of time of not detecting any user interactions, the
// console will return to the default sttae of auto-updating based on
// latest events.
//
// The Iframes themselves will display special zoneminder "views", that
// are adaptations of the "classic" ZoneMinder views to suit the ARC
// console. Specifically:
//
//   o arc-events-latest - list of latest events
//   o arc-events - text list of latest events
//   o arc-event - video view of one event
//   o arc-watch - video view of one live feed
//   o arc-montage - video view of all live feeds
//
// The 'arc-console' view serves as the main controller.  It periodcially
// updates the "arc-events" iframe, where that view will percolate
// up to the console the event id of the most recent event.  If the
// console detects this is a different new event from the last time, then
// if decides what, if any, of the other iframes will need to be updated
// based on the current state.

//--------------------
// Asynchronous Timers
//
//   o event list update timer - triggers updates to the arc-events iframe
//     view
//
//   o live view cycle timer - active only if user has selected to
//     cycle through the monitors' live feeds
//
//   o user interaction timer - set (or reset) when the user performs an
//     interaction with the console that is important for changing the
//     auto-updating behavior.  When the timer expires, normal
//     auto-updating resumes.
//
//   o new event highlight timer - used to temporarily provide a visual
//     indication of a newly arrived event, especially in case where
//     user is interacting with the console and automatic refresh of the
//     iframes is suppressed.

//--------------------
// State Machine

// Because there are a few types of asynchronous timer-based triggers, and
// user actions, the state space of the console is non-trivial.  Tracking
// the current state and how the state should transition to the next state
// based upon the triggers and actions is needed to make sure that console
// is helpful with automatic updates, but non-invasive when the user is
// interacting with it.  This is the main complexity in this module and we
// keep a number of "state" variables so we can track the important things.

//--------------------
// Screen layout and sizing

// This console attempts to fill the entire available browser window size,
// so sets the main HTML elements' styles after the page loads from within
// JavaScript.  This removes some of the layout control from the CSS file,
// though we have tried to keep it to the minimum.  The general design
// decision here was tailoring this to a full screen display console, so
// video sizes are scaled up or down as needed to fill as much of the
// space as possible.

//========================================
// Constants
//========================================

//--------------------
// HTML tag Ids for referencing and manipulating

// Screen has two main panes that live side-by-side: one for events and one
// for live feeds.
//
var ARC_EVENT_PANE_ID = "arc-event-pane";
var ARC_LIVE_PANE_ID = "arc-live-pane";

// The main iframes/views that the console controls
var ARC_EVENT_LIST_LATEST_IFRAME_ID = "arc-event-list-latest-iframe";
var ARC_EVENT_LIST_IFRAME_ID = "arc-event-list-iframe";
var ARC_EVENT_VIDEO_IFRAME_ID = "arc-event-video-iframe";
var ARC_LIVE_VIDEO_IFRAME_ID = "arc-live-video-iframe";

// These are statically drawn UI element to control the events and live
// feed.  We need to find out how much screen real estate these take when
// deciding how big to make the iframes and video streams
//
var ARC_EVENT_CONTROLS_ID = "arc-event-controls";
var ARC_LIVE_CONTROLS_ID = "arc-live-controls";

// We give a toggle ability to show more or less events in the event list,
// so we need to adjust the css classes to show/hide the control buttons as
// appropriate.
var ARC_LESS_EVENTS_CONTROL_ID = "arc-event-less";
var ARC_MORE_EVENTS_CONTROL_ID = "arc-event-more";

// For alerting
var ARC_TURN_ALERTS_ON_BUTTON_ID = 'arc-turn-alerts-on';
var ARC_TURN_ALERTS_OFF_BUTTON_ID = 'arc-turn-alerts-off';

// For alert audio and control buttons
var ARC_ALERT_AUDIO_ID = 'arc-alert-audio';
var ARC_TURN_SOUND_ON_BUTTON_ID = 'arc-turn-sound-on';
var ARC_TURN_SOUND_OFF_BUTTON_ID = 'arc-turn-sound-off';

//========================================
// State variables
//========================================

// Has the user recently interacted with the UI in an important way?
var ARC_RecentUserInteraction = false;

// Is the event video iframe showing something from an automatic update?
// (Else is showing something the user selected.)
var ARC_EventRefreshAutomatic = true;

// We do not auto-refresh the event list and video if the user is
// intertacting with it. So new events arriving might not be reflected in
// the UI display.  This is set when that exists so that when the user is
// done interacting, the screen can be refreshed.
var ARC_EventRefreshPending = false;

// Is the live video iframe showing something from an automatic update?
// (Else is showing something the user selected.)
var ARC_LiveRefreshAutomatic = true;

// Whether or not the user has chosen to cycle through the live
// feeds from all the monitors.
var ARC_LiveVideoCycling = false;

// What event is latest event from the arc-events-latest iframe callback
var ARC_LatestEventId = null;
var ARC_LatestEventHasEnded = true;

// When live video is showing the montage of all monitors (a.k.a.,
// cameras), the ARC_CurrentMonitorId will be set to this value.
var ARC_ALL_MONITOR_ID = "ALL";

// What camera is currently being viewed in the live feed video iframe
var ARC_CurrentMonitorId = null;

// The event id of the event video being played in the event video iframe
var ARC_CurrentVideoEventId = null;

// This gets set after we get the first callback from the arc-events-latest
// iframe load to indicate that we have launched the console.
//
var ARC_FirstCallbackHandled = false;

// User can toggle this on and off.
var ARC_AlertsEnabled = true;

// How many events to show in the event list table and the values used
// for "more" and "less".
//
var ARC_EVENT_LIST_LIMIT_LESS = 10;
var ARC_EVENT_LIST_LIMIT_MORE = 50;
var ARC_EventListLimit = ARC_EVENT_LIST_LIMIT_LESS;

//========================================
// Timers
//========================================

//--------------------
// Timer for updating the event listings

var ARC_EVENT_LIST_TIMER_INTERVAL_MS = 15000;

var ARC_EventListTimer = null;

function ARC_startEventListTimer() {
    ARC_clearEventListTimer();
    ARC_EventListTimer = setTimeout( ARC_loadEventListLatestIframe, 
							  ARC_EVENT_LIST_TIMER_INTERVAL_MS );

    ARC_logEvent( "Event list timer started.");
}

function ARC_clearEventListTimer() {
    if ( ARC_EventListTimer == null )
	   return;
    clearTimeout( ARC_EventListTimer );
    ARC_EventListTimer = null;

    ARC_logEvent( "Event list timer cleared.");
}


//--------------------
// Timer for when the user selects to cycle through the monitors in
// the live feed video iframe.

// Normal interval between changing monitors when cycling through them.
var ARC_LIVE_CYCLE_TIMER_INTERVAL_MS = 20000;

// If we are cycling through videos and a new event arrives, we will change
// the display to the monitor for that event for this time period, and then
// resume cycling from that id forward.
var ARC_LIVE_CYCLE_NEW_EVENT_TIMER_INTERVAL_MS = 60000;

var ARC_LiveCycleTimer = null;

function ARC_startLiveCycleTimer( intervalMs ) {
    ARC_clearLiveCycleTimer();

    if ( ! intervalMs )
	   intervalMs = ARC_LIVE_CYCLE_TIMER_INTERVAL_MS;

    ARC_LiveCycleTimer = setTimeout( ARC_doLiveCycleNext, intervalMs );

    ARC_logEvent( "Live feed cycle timer started. ("+intervalMs+"ms)");
}

function ARC_clearLiveCycleTimer() {
    if ( ARC_LiveCycleTimer == null )
	   return;
    clearTimeout( ARC_LiveCycleTimer );
    ARC_LiveCycleTimer = null;

    ARC_logEvent( "Live feed cycle timer cleared.");
}

//--------------------
// Timer for tracking the last time a user interacted with the console
// in a way that will make us want to alter the behavior of what will
// be automatically updated.

var ARC_USER_ACTIVE_TIMEOUT_MS = 30000;

var ARC_UserActivityTimer = null;

function ARC_startUserActivityTimer() {
    ARC_clearUserActivityTimer();
    ARC_UserActivityTimer = setTimeout( ARC_setUserInactive, 
								ARC_USER_ACTIVE_TIMEOUT_MS );

    ARC_logEvent( "User activity timer started.");
}

function ARC_clearUserActivityTimer() {
    if ( ARC_UserActivityTimer == null )
	   return;
    clearTimeout( ARC_UserActivityTimer );
    ARC_UserActivityTimer = null;
    ARC_logEvent( "User activity timer cleared.");
}

function ARC_setUserInactive() {
    ARC_logEvent( "User activity timer expired.");

    ARC_clearUserActivityTimer();
    ARC_EventRefreshAutomatic = true;
    ARC_LiveRefreshAutomatic = true;

    if ( ARC_EventRefreshPending ) {
	   ARC_logEvent( "Refreshing due to pending event.");
	   ARC_loadEventListIframe();		  
	   ARC_loadEventVideoIframe( ARC_LatestEventId );
    }
}

//========================================
// New Event Highlight Notification Timer and Logic
//========================================

// Timer for new event highlighting

// The HTML elementid we will toggle a css class on to highlight
var ARC_NEW_EVENT_HIGHLIGHT_ID = "page";

// The class we add and remove to toggle the highlighting
var ARC_NEW_EVENT_HIGHLIGHT_CLASS = "arc-new-event-highlight";

// How long do we continue the highlighting for new events
var ARC_NEW_EVENT_HIGHLIGHT_DURATION_MS = 10000;

// This controls the rate of "flashing" for the visual notifcation
var ARC_NEW_EVENT_HIGHLIGHT_INTERVAL_MS = 250;

var ARC_NewEventHighlightTimer = null;
var ARC_NewEventFlashingTimer = null;

var ARC_NewEventHighlightState = false;

function ARC_startNewEventHighlight() {

    if ( ! ARC_AlertsEnabled )
	   return;

    ARC_toggleNewEventHighlight();

    ARC_clearNewEventHighlightTimer();
    ARC_NewEventHighlightTimer = setTimeout( ARC_cancelNewEventHighlight, 
									ARC_NEW_EVENT_HIGHLIGHT_DURATION_MS );

    ARC_startAlarmSound();

    ARC_logEvent( "New event highlight timer started.");
}

function ARC_toggleNewEventHighlight() {

    if ( ARC_NewEventHighlightState )
	   $(ARC_NEW_EVENT_HIGHLIGHT_ID).removeClass(ARC_NEW_EVENT_HIGHLIGHT_CLASS);
    else
	   $(ARC_NEW_EVENT_HIGHLIGHT_ID).addClass(ARC_NEW_EVENT_HIGHLIGHT_CLASS);
	   
    ARC_NewEventHighlightState = ! ARC_NewEventHighlightState;

    ARC_clearNewEventFlashingTimer();
    ARC_NewEventFlashingTimer = setTimeout( ARC_toggleNewEventHighlight,
								    ARC_NEW_EVENT_HIGHLIGHT_INTERVAL_MS );

}

function ARC_cancelNewEventHighlight() {
    ARC_stopAlarmSound();
    ARC_clearNewEventHighlightTimer();
    ARC_clearNewEventFlashingTimer();
    $(ARC_NEW_EVENT_HIGHLIGHT_ID).removeClass(ARC_NEW_EVENT_HIGHLIGHT_CLASS);
    ARC_NewEventHighlightState = false;
    ARC_logEvent( "New event highlight timer cancelled.");
}

function ARC_clearNewEventHighlightTimer() {
    if ( ARC_NewEventHighlightTimer == null )
	   return;
    clearTimeout( ARC_NewEventHighlightTimer );
    ARC_NewEventHighlightTimer = null;
}

function ARC_clearNewEventFlashingTimer() {
    if ( ARC_NewEventFlashingTimer == null )
	   return;
    clearTimeout( ARC_NewEventFlashingTimer );
    ARC_NewEventFlashingTimer = null;
}

//--------------------
// New event alarm sound

var ARC_SoundEnabled = true;

function ARC_enableSound() {
    ARC_SoundEnabled = true;
    document.getElementById(ARC_TURN_SOUND_ON_BUTTON_ID).addClass('arc-hidden');
    document.getElementById(ARC_TURN_SOUND_OFF_BUTTON_ID).removeClass('arc-hidden');
}
function ARC_disableSound() {
    ARC_SoundEnabled = false;
    ARC_stopAlarmSound( );
    document.getElementById(ARC_TURN_SOUND_OFF_BUTTON_ID).addClass('arc-hidden');
    document.getElementById(ARC_TURN_SOUND_ON_BUTTON_ID).removeClass('arc-hidden');
}

function ARC_startAlarmSound( ) {
    if ( ! SOUND_ON_ALARM )
	   return;

    if ( ! ARC_SoundEnabled )
	   return;

    document.getElementById(ARC_ALERT_AUDIO_ID).play();
}

function ARC_stopAlarmSound( ) {
    if ( ! SOUND_ON_ALARM )
	   return;

    document.getElementById(ARC_ALERT_AUDIO_ID).pause();
}

//========================================
// Main Functions
//========================================

//--------------------
// Called on the page body onload event
function ARC_handlePageLoaded( ) {
    ARC_setConsoleLayout();
    ARC_loadEventListLatestIframe()
}

//========================================
// Event Pane Interactions
//========================================

// Loads the (hidden) latest event list into an iframe.  The "onload"
// callback of the page loaded will relay back to this module via the
// ARC_handleEventListLatestPageLoaded() handler.
//
function ARC_loadEventListLatestIframe() {
    ARC_loadIframe( ARC_EVENT_LIST_LATEST_IFRAME_ID, 
				'?view=arc-events-latest' );
    
    ARC_logEvent("Loading latest events iframe");
}

//--------------------
// Called by the arc-events-latest (child) view code when it has loaded in
// its iframe. Passing in the current latest event id and the monitor the
// event is tied to.
//
// The 'hasEnded' flag is important since the video of the event is not
// ready until it has ended.  We will still want to alert and switch the
// live feed to the view where the event is occurring, but we will want to
// later updated the evevent video witht he event once it has ended.  Note
// that if we alerted the user to the start of the event, then we will not
// want to alert the user again when it ends, since a double alert would be
// annoying.
//
function ARC_handleEventListLatestPageLoaded( eventId, monitorId, hasEnded ) {

    try {
	   ARC_logEvent("Handling latest events iframe loaded. eventId="+eventId
				 +", monitorId="+monitorId+", hasEnded="+hasEnded);

	   if ( ! eventId ) {
		  if ( ! ARC_FirstCallbackHandled )
			 ARC_handleNoEventsOnStartup();
		  return;
	   } // if no event id

	   // No need to update any other iframes if there are no new events.
	   if (( eventId == ARC_LatestEventId )
		  && ( ARC_LatestEventHasEnded == hasEnded )) {
		  ARC_logEvent("Same event as last time.");
		  return;
	   }

	   // Always give some extra visual cue about a newly arrived event.
	   // It is very important if the user is interacting and we did not
	   // auto-refesh the iframes, but it is also useful even when the
	   // iframes are udpated.  The one exception is when we first load
	   // the page.  Definitionally, this always has a new event, but we
	   // want to suppress the visual highlighing in that case.
	   if (( ARC_LatestEventId != null )
		  && ( eventId != ARC_LatestEventId ))
		  ARC_startNewEventHighlight();
	   
	   ARC_LatestEventId = eventId;
	   ARC_LatestEventHasEnded = hasEnded;

	   // Only re-load event-related if use has not been recently 
	   // interacting with them.
	   if ( ARC_EventRefreshAutomatic ) {
		  ARC_logEvent("Event refresh automatic. Loading event iframes.");
		  ARC_loadEventListIframe();		  
		  ARC_loadEventVideoIframe( eventId );
		  ARC_EventRefreshPending = false;
	   }

	   // Need to remember that we did not updater the event list with
	   // the latest events so we can update it when the user 
	   // has not interacted with the console for a while.
	   else {
		  ARC_logEvent("Event iframes not auto-loaded.");
		  ARC_EventRefreshPending = true;
	   }

	   // Only re-load the live video with the recent event's monitor if
	   // the user has not recently been interacting with the live video
	   // are.  We also do not switch to the single camera view if we are
	   // seeing the full montage (since we definitionally will already be
	   // looking at the live feed of the event.)
	   //
	   if ( ARC_LiveRefreshAutomatic
		   && ( ARC_CurrentMonitorId != ARC_ALL_MONITOR_ID )) {
		  ARC_logEvent("Live refresh automatic. Loading live iframes.");
		  ARC_resetLiveCycleIfNeeded();
		  ARC_changeCurrentMonitorId( monitorId );
	   }
	   else {
		  ARC_logEvent("Live iframes not auto-loaded.");
	   }

    }
    catch (e) {
	   // FIXME: Error handling
    }
    finally {
	   ARC_FirstCallbackHandled = true;
	   ARC_startEventListTimer();
    }
}

//--------------------
function ARC_handleNoEventsOnStartup() {
    ARC_loadEventListIframe();		  
    ARC_changeCurrentMonitorId( ARC_MonitorIdList[0] );
}

//--------------------
function ARC_handleEventRowClick( eventId ) {

    ARC_logEvent("Event row click handled. eventId="+eventId);

    // User interaction will temporarily stop auto-updating of event list
    // and event video
    ARC_EventRefreshAutomatic = false;
    ARC_startUserActivityTimer();

    ARC_loadEventVideoIframe( eventId );
}

//--------------------
function ARC_loadEventListIframe( ) {

    ARC_loadIframe( ARC_EVENT_LIST_IFRAME_ID,
				'?view=arc-events&limit='+ARC_EventListLimit );
}

//--------------------
function ARC_loadEventVideoIframe( eventId ) {

    if ( ! eventId )
	   return;

    ARC_CurrentVideoEventId = eventId;

    ARC_reloadCurrentEventVideo( );
}

//--------------------
function ARC_reloadCurrentEventVideo( ) {

    if ( ! ARC_CurrentVideoEventId )
	   return;

    var viewEventSrc = '?view=arc-event&eid='+ARC_CurrentVideoEventId;
    viewEventSrc += "&videoWidth="+ARC_EventVideoSize['width']
	   +"&videoHeight="+ARC_EventVideoSize['height'];

    ARC_loadIframe( ARC_EVENT_VIDEO_IFRAME_ID, viewEventSrc );
}

//--------------------
// Calls to this are relayed from the arc-event.js code in the iframe
function ARC_handleEventVideoClicked() {
    ARC_reloadCurrentEventVideo( );
}

//--------------------
function ARC_handleLiveNameClick( monitorId ) {

    if ( ! monitorId )
	   return;

    ARC_logEvent("Live monitor click handled. monitorId="+monitorId);

    ARC_cancelLiveCycle();

    if ( monitorId != ARC_ALL_MONITOR_ID )
    {
	   // User interaction will temporarily stop auto-updating of event list
	   // and event video. This does not appy for view all monitors.
	   ARC_LiveRefreshAutomatic = false;
	   ARC_startUserActivityTimer();
    }

    ARC_changeCurrentMonitorId( monitorId );
}

//--------------------
function ARC_changeCurrentMonitorId( monitorId ) {
//    alert( "ARC_changeCurrentMonitorId: monitorId="+monitorId+", viewMonitorSrc="+viewMonitorSrc );

    if ( monitorId == ARC_CurrentMonitorId )
	   return;

    // Make all rows inactive first, the make the current one
    // active
    if ( ARC_CurrentMonitorId )
	   $('arc-live-button-'+ARC_CurrentMonitorId).removeClass( 'arc-button-active' );
    $('arc-live-button-'+monitorId).addClass( 'arc-button-active' );

    ARC_CurrentMonitorId = monitorId;

    ARC_reloadCurrentLiveVideo( );
}

//--------------------
function ARC_reloadCurrentLiveVideo( ) {

    if ( ! ARC_CurrentMonitorId )
	   return;

    var viewMonitorSrc;
    if ( ARC_CurrentMonitorId == ARC_ALL_MONITOR_ID )
    {
	   viewMonitorSrc = '?view=arc-montage';
	   viewMonitorSrc += "&videoWidth="+ARC_LiveVideoSize['width']
		  +"&videoHeight="+ARC_LiveVideoSize['height'];
    }
    else
    {
	   viewMonitorSrc = '?view=arc-watch&mid='+ARC_CurrentMonitorId;
	   viewMonitorSrc += "&videoWidth="+ARC_LiveVideoSize['width']
		  +"&videoHeight="+ARC_LiveVideoSize['height'];
    }

    ARC_loadIframe( ARC_LIVE_VIDEO_IFRAME_ID, viewMonitorSrc );
}

//--------------------
function ARC_handleLiveCycleClick() {

    ARC_startNewEventHighlight();
    return;

    if ( ARC_LiveVideoCycling )
	   return;

    // If we are viewing the montage of all camera, then switch to the
    // first one before beginning cycling.
    if ( ARC_CurrentMonitorId == ARC_ALL_MONITOR_ID )
	   ARC_changeCurrentMonitorId( ARC_MonitorIdList[0] );

    ARC_startLiveCycleTimer();
    ARC_LiveVideoCycling = true;

    // We define cycling as the user reliquishing control to allow any
    // incoming new events to switch the live feed.
    ARC_EventRefreshAutomatic = true;

    $('arc-live-button-CYCLE').addClass( 'arc-button-active' );

}

//--------------------
function ARC_cancelLiveCycle() {
    ARC_clearLiveCycleTimer();
    ARC_LiveVideoCycling = false;

    $('arc-live-button-CYCLE').removeClass( 'arc-button-active' );
}

//--------------------
function ARC_doLiveCycleNext() {

    var curIndex = -1;
    for ( var i = 0; i < ARC_MonitorIdList.length; i++ ) {
	   if ( ARC_CurrentMonitorId != ARC_MonitorIdList[i] )
		  continue;
	   curIndex = i;
	   break;
    }
    
    // Note that we start at '0' if we did not find the monitor id in the
    // list.
    var nextIndex = curIndex + 1;
    if ( nextIndex >= ARC_MonitorIdList.length )
	   nextIndex = 0;

    ARC_changeCurrentMonitorId( ARC_MonitorIdList[nextIndex] );
    
    ARC_startLiveCycleTimer();
}

//--------------------
// Called when we update the live video for an event. If live cycling
// is enbled it will resume after a slightly longer display of the
// live feed from the event's monitor.
function ARC_resetLiveCycleIfNeeded() {
    if ( ! ARC_LiveVideoCycling )
	   return;

    ARC_clearLiveCycleTimer();
    ARC_startLiveCycleTimer( ARC_LIVE_CYCLE_NEW_EVENT_TIMER_INTERVAL_MS );
}

//--------------------
function ARC_forceEventRefresh() {
    ARC_setUserInactive();
}

//--------------------
function ARC_showMoreEvents() {

    ARC_EventListLimit = ARC_EVENT_LIST_LIMIT_MORE;
    $(ARC_MORE_EVENTS_CONTROL_ID).addClass("arc-hidden");
    $(ARC_LESS_EVENTS_CONTROL_ID).removeClass("arc-hidden");
    ARC_forceEventListRefresh();

}

//--------------------
function ARC_showLessEvents() {

    ARC_EventListLimit = ARC_EVENT_LIST_LIMIT_LESS;
    $(ARC_LESS_EVENTS_CONTROL_ID).addClass("arc-hidden");
    $(ARC_MORE_EVENTS_CONTROL_ID).removeClass("arc-hidden");
    ARC_forceEventListRefresh();
}

//--------------------
function ARC_forceEventListRefresh() {

    // We consider this a user action that suspends automatic updates
    // of the event pane
    ARC_EventRefreshAutomatic = false;
    ARC_startUserActivityTimer();

    ARC_loadEventListIframe();		  
}

//--------------------
function ARC_disableAlerts() {
    ARC_AlertsEnabled = false;
    ARC_cancelNewEventHighlight();
    document.getElementById(ARC_TURN_ALERTS_OFF_BUTTON_ID).addClass('arc-hidden');
    document.getElementById(ARC_TURN_ALERTS_ON_BUTTON_ID).removeClass('arc-hidden');

}

//--------------------
function ARC_enableAlerts() {
    ARC_AlertsEnabled = true;
    document.getElementById(ARC_TURN_ALERTS_ON_BUTTON_ID).addClass('arc-hidden');
    document.getElementById(ARC_TURN_ALERTS_OFF_BUTTON_ID).removeClass('arc-hidden');

}

//========================================
// Window Size and Layout Management
//========================================

var ARC_ConsoleLayoutSet = false;

// If we detect a window smaller than these dimensions, we will use these
var ARC_MIN_WINDOW_WIDTH = 700;
var ARC_MIN_WINDOW_HEIGHT = 500;

// What percentage of the screen the main areas takes up in both
// portrait and landscape modes
var ARC_LIVE_PANE_WIDTH_SCALE_LANDSCAPE = 0.64;
var ARC_LIVE_PANE_HEIGHT_SCALE_LANDSCAPE = 0.99;
var ARC_EVENT_PANE_WIDTH_SCALE_LANDSCAPE = 0.34;
var ARC_EVENT_PANE_HEIGHT_SCALE_LANDSCAPE = 0.99;

var ARC_LIVE_PANE_WIDTH_SCALE_PORTRAIT = 0.99;
var ARC_LIVE_PANE_HEIGHT_SCALE_PORTRAIT = 0.59;
var ARC_EVENT_PANE_WIDTH_SCALE_PORTRAIT = 0.99;
var ARC_EVENT_PANE_HEIGHT_SCALE_PORTRAIT = 0.39;

var ARC_EVENT_LIST_HEIGHT_SCALE = 0.35;

// Use this to make some siggle room in calculations. These (unfortunately)
// need to be in sync with arc-console.css.
var ARC_PAGE_MARGIN_PIXELS = 5;
var ARC_PANE_MARGIN_PIXELS = 2;
var ARC_IFRAME_MARGIN_PIXELS = 2;

// Where we store the video sizes to fit layout to screen size
var ARC_WindowSize = {};
var ARC_EventVideoSize = {};
var ARC_LiveVideoSize = {};

//--------------------
// Returns a two elemement array: [ width, height ]
function ARC_setConsoleLayout() {

    if ( ARC_ConsoleLayoutSet )
	   return;
    ARC_ConsoleLayoutSet = true;

    ARC_WindowSize = ARC_getBrowserWindowSize();

    // Adjust for min sizes and margins before calculating pane sizes
    var winSize = ARC_WindowSize;
    if ( winSize['width'] < ARC_MIN_WINDOW_WIDTH )
	   winSize['width'] = ARC_MIN_WINDOW_WIDTH;
    if ( winSize['height'] < ARC_MIN_WINDOW_HEIGHT )
	   winSize['height'] = ARC_MIN_WINDOW_HEIGHT;

    winSize['width'] -= 2 * ARC_PAGE_MARGIN_PIXELS;
    winSize['height'] -= 2 * ARC_PAGE_MARGIN_PIXELS;

    var lpWidthScale = ARC_LIVE_PANE_WIDTH_SCALE_LANDSCAPE;
    var lpHeightScale = ARC_LIVE_PANE_HEIGHT_SCALE_LANDSCAPE;
    var epWidthScale = ARC_EVENT_PANE_WIDTH_SCALE_LANDSCAPE;
    var epHeightScale = ARC_EVENT_PANE_HEIGHT_SCALE_LANDSCAPE;

    // Check for portrait layout overrides
    if ( winSize['width'] < winSize['height'] ) {
	   lpWidthScale = ARC_LIVE_PANE_WIDTH_SCALE_PORTRAIT;
	   lpHeightScale = ARC_LIVE_PANE_HEIGHT_SCALE_PORTRAIT;
	   epWidthScale = ARC_EVENT_PANE_WIDTH_SCALE_PORTRAIT;
	   epHeightScale = ARC_EVENT_PANE_HEIGHT_SCALE_PORTRAIT;
    }

    var elHeightScale = ARC_EVENT_LIST_HEIGHT_SCALE;

    var ecHeight = document.getElementById(ARC_EVENT_CONTROLS_ID).clientHeight;
    var lcHeight = document.getElementById(ARC_LIVE_CONTROLS_ID).clientHeight;

    var elem;

    elem = document.getElementById(ARC_EVENT_PANE_ID);
    elem.style.width = Math.floor(winSize['width'] * epWidthScale)+"px";

    elem = document.getElementById(ARC_LIVE_PANE_ID);
    elem.style.width = Math.floor(winSize['width'] * lpWidthScale)+"px";

    elem = document.getElementById(ARC_EVENT_LIST_IFRAME_ID);
    elem.style.height = Math.floor(winSize['height'] * epHeightScale * elHeightScale)+"px";

    var eventIframeWidth = Math.floor(winSize['width'] * epWidthScale) - ( 2 * ARC_PANE_MARGIN_PIXELS );
    var eventIframeHeight = Math.floor(winSize['height'] * epHeightScale * (1.0 - elHeightScale)) - ecHeight - ( 2 * ARC_PANE_MARGIN_PIXELS );

    ARC_EventVideoSize['width'] = eventIframeWidth - ( 2 * ARC_IFRAME_MARGIN_PIXELS );
    ARC_EventVideoSize['height'] = eventIframeHeight - ( 2 * ARC_IFRAME_MARGIN_PIXELS );

    elem = document.getElementById(ARC_EVENT_VIDEO_IFRAME_ID);
    elem.style.width = eventIframeWidth+"px";
    elem.style.height = eventIframeHeight+"px";

    var liveIframeWidth = Math.floor(winSize['width'] * lpWidthScale) - ( 2 * ARC_PANE_MARGIN_PIXELS );
    var liveIframeHeight = Math.floor( winSize['height'] * lpHeightScale ) - lcHeight - ( 2 * ARC_PANE_MARGIN_PIXELS );

    ARC_LiveVideoSize['width'] = liveIframeWidth - ARC_IFRAME_MARGIN_PIXELS;
    ARC_LiveVideoSize['height'] = liveIframeHeight - ARC_IFRAME_MARGIN_PIXELS;

    elem = document.getElementById(ARC_LIVE_VIDEO_IFRAME_ID);
    elem.style.width = liveIframeWidth+"px";
    elem.style.height = liveIframeHeight+"px";

    ARC_showWindowInfo();   
}

//--------------------
// Returns a hash: [ "width": width, "height": height ]
function ARC_getBrowserWindowSize() {

    // Adapted from: http://www.javascripter.net/faq/browserw.htm

    var winW = 630, winH = 460;
    if (document.body && document.body.offsetWidth) {
	   winW = document.body.offsetWidth;
	   winH = document.body.offsetHeight;
    }
    if (document.compatMode=='CSS1Compat' &&
	   document.documentElement &&
	   document.documentElement.offsetWidth ) {
	   winW = document.documentElement.offsetWidth;
	   winH = document.documentElement.offsetHeight;
    }
    if (window.innerWidth && window.innerHeight) {
	   winW = window.innerWidth;
	   winH = window.innerHeight;
    }

    var size = {};
    size['width'] = winW;
    size['height'] = winH;
    return size;
}

//========================================
// Window Re-size handling
//========================================

var ARC_WINDOW_RESIZE_DELAY_MS = 2000;

var ARC_WindowResizeTimer = null;

//--------------------
function ARC_handleWindowResizeEvent() {

    if ( ARC_WindowResizeTimer != null )
	   clearTimeout( ARC_WindowResizeTimer );

    ARC_WindowResizeTimer = setTimeout( ARC_handleWindowResizeDone,
								ARC_WINDOW_RESIZE_DELAY_MS );
}

//--------------------
function ARC_handleWindowResizeDone() {

    ARC_logEvent( "Window resize timer now done." );

    clearTimeout( ARC_WindowResizeTimer );
    ARC_WindowResizeTimer = null;

    ARC_ConsoleLayoutSet = false;
    ARC_setConsoleLayout();

    // The videos that are currently rendered and playing are not resized
    // easily, so easiest to just reload them to get them to adjust to
    // the correct size for the new layout.
    ARC_reloadCurrentLiveVideo( );
    ARC_reloadCurrentEventVideo( );

}

// Main deal to get hooked intot he resize events.
window.onresize = ARC_handleWindowResizeEvent;

// Found I also needed this for iPad/Safari
window.onorientationchange = ARC_handleWindowResizeEvent;

//========================================
// Utility Functions
//========================================

//--------------------
function ARC_loadIframe( iframeId, urlQueryParams ) {
//    alert( "LOAD! iframeId=" + iframeId + ", urlQueryParams=" + urlQueryParams );

    var iframe = document.getElementById(iframeId);
    if ( ! iframe ) {
	alert( "Cannot find iframe with id="+iframeId );
	return;
    }

    iframe.setAttribute('src', urlQueryParams );

}

//--------------------
function ARC_logEvent( msg, showStateInfo ) {
    if ( ! ARC_DEBUG_ENABLED )
	   return;

    if ( ! console || ! console.log )
	   return;

    if ( showStateInfo ) {
	   var infoList = ARC_getStateInfoList();
	   console.log( "ARC: " + msg + "\n     " + infoList.join( "\n     " ));
    }
    else {
	   console.log( "ARC: " + msg );
    }
}

//--------------------
function ARC_showWindowInfo( ) {

    var msgList = [];
    
    msgList.push( "UserAgent="+navigator.userAgent);
    msgList.push( "ARC_WindowSize="+ARC_WindowSize['width']+"x"+ARC_WindowSize['height'] );
    msgList.push( "ARC_EventVideoSize="+ARC_EventVideoSize['width']+"x"+ARC_EventVideoSize['height'] );
    msgList.push( "ARC_LiveVideoSize="+ARC_LiveVideoSize['width']+"x"+ARC_LiveVideoSize['height'] );

    ARC_logEvent( "ARC: Window Info\n     " + msgList.join( "\n     " ));
}


//--------------------
function ARC_getStateInfoList( ) {
    
    var msgList = [];

    msgList.push( "ARC_RecentUserInteraction="+(ARC_RecentUserInteraction?"true":"false") );
    msgList.push( "ARC_EventRefreshAutomatic="+(ARC_EventRefreshAutomatic?"true":"false") );
    msgList.push( "ARC_LiveRefreshAutomatic="+(ARC_LiveRefreshAutomatic?"true":"false") );
    msgList.push( "ARC_LiveVideoCycling="+(ARC_LiveVideoCycling?"true":"false") );

    msgList.push( "ARC_LatestEventId="+ARC_LatestEventId );
    msgList.push( "ARC_LatestEventHasEnded="+ARC_LatestEventHasEnded );
    msgList.push( "ARC_CurrentMonitorId="+ARC_CurrentMonitorId );

    msgList.push( "ARC_EventListTimer="+(ARC_EventListTimer?"true":"false") );
    msgList.push( "ARC_LiveCycleTimer="+(ARC_LiveCycleTimer?"true":"false") );
    msgList.push( "ARC_UserActivityTimer="+(ARC_UserActivityTimer?"true":"false") );

    return msgList;

}